# Data integration and metadata annotation

Repository for integrating data in HDF5 from various sources and managing metadata updates in the integrated files. 

Includes tools and workflows for comprehensive data integration and automated metadata review processes.

## Installation

Follow these steps to install and set up the project:

1. Download our GitLab repository in your GitLab folder, or alternatively open a Git Bash terminal and run the following commands: 
  ```
  cd Path/to/GitLab
  git clone https://gitlab.psi.ch/5505/data-integration-and-metadata-annotation.git
  ```

2. Open an Anaconda Prompt (Anaconda3) as administrator, and set the current directory to the path of the project's folder.

3. Create the project's environment `multiphase_chemistry_env` by running the following command:
  ```
  conda env create -f environment.yml
  ```

### Working with Jupyter Notebook on the `multiphase_chemistry_env`

1. Open an Anaconda Prompt as a regular user, ensure that `multiphase_chemistry_env` is in the list of available enviroments and activate it by running the following commands:
  ```
  conda env list
  conda activate multiphase_chemistry_env
  ```
2. Register the associated kernel in Jupyter by running:

  ```
  python -m ipykernel install --user --name multiphase_chemistry_env --display-name "Python (multiphase_chemistry_env)"  
  ```
3. Start a Jupyter Notebook by running the command:
  ```
  jupyter notebook
  ```
and select the `multiphase_chemistry_env` environment from the kernel options.

### Working with Visual Studio Code (VS Code) on the `multiphase_chemistry_env`
1. Open the project in VS Code, click on the Python interpreter in the status bar and choose the `multiphase_chemistry_env` environment.

## Data integration workflow

## Metadata review workflow
    - review through branches
    - updating files with metadata in Openbis
    
### Specifying a compound attribute in yaml language.
Consider the compound attribute *relative_humidity*, which has subattributes *value*, *units*, *range*, and *definition*. The yaml description of
such an attribute is as follows:
```yaml
relative_humidity:
  value: 65
  units: percentage
  range: '[0,100]'
  definition: 'Relative humidity represents the amount of water vapor present in the air relative to the maximum amount of water vapor the air can hold at a given temperature.'  
```
### Deleting or renaming a compound attribute in yaml language.
  - Assume the attribute *relative_humidity* already exists. Then it should be displayed as follows with the subattribute *rename_as*. This can be set differently to suggest a renaming of the attribute. 
  - To suggest deletion of an attribute, we are required to add a subattribute *delete* with value as *true*. Below for example, the
  attribute *relative_ humidity* is suggested to be deleted. Otherwise if *delete* is set as *false*, it will have no effect.
```yaml
relative_humidity:
  delete: true # we added this line in the review process
  rename_as: relative_humidity
  value: 65
  units: percentage
  range: '[0,100]'
  definition: 'Relative humidity represents the amount of water vapor present in the air relative to the maximum amount of water vapor the air can hold at a given temperature.'

```

## -------------------
## Getting started

To make it easy for you to get started with GitLab, here's a list of recommended next steps.

Already a pro? Just edit this README.md and make it your own. Want to make it easy? [Use the template at the bottom](#editing-this-readme)!

## TODO
* Talk to Thorsten about rga txt files. Example folder contains incosistent rga txt files, there is no unique column separator (, or )

## Add your files

- [ ] [Create](https://docs.gitlab.com/ee/user/project/repository/web_editor.html#create-a-file) or [upload](https://docs.gitlab.com/ee/user/project/repository/web_editor.html#upload-a-file) files
- [ ] [Add files using the command line](https://docs.gitlab.com/ee/gitlab-basics/add-file.html#add-a-file-using-the-command-line) or push an existing Git repository with the following command:

```
cd existing_repo
git remote add origin https://gitlab.psi.ch/5505/functionspython.git
git branch -M main
git push -uf origin main
```


```
cd existing_repo
git remote add origin https://gitlab.psi.ch/5505/functionspython.git
git branch -M main
git push -uf origin main
```

## Integrate with your tools

## Integrate with your tools

- [ ] [Set up project integrations](https://gitlab.psi.ch/5505/functionspython/-/settings/integrations)

## Collaborate with your team

- [ ] [Invite team members and collaborators](https://docs.gitlab.com/ee/user/project/members/)
- [ ] [Create a new merge request](https://docs.gitlab.com/ee/user/project/merge_requests/creating_merge_requests.html)
- [ ] [Automatically close issues from merge requests](https://docs.gitlab.com/ee/user/project/issues/managing_issues.html#closing-issues-automatically)
- [ ] [Enable merge request approvals](https://docs.gitlab.com/ee/user/project/merge_requests/approvals/)
- [ ] [Automatically merge when pipeline succeeds](https://docs.gitlab.com/ee/user/project/merge_requests/merge_when_pipeline_succeeds.html)

## Test and Deploy

Use the built-in continuous integration in GitLab.

- [ ] [Get started with GitLab CI/CD](https://docs.gitlab.com/ee/ci/quick_start/index.html)
- [ ] [Analyze your code for known vulnerabilities with Static Application Security Testing(SAST)](https://docs.gitlab.com/ee/user/application_security/sast/)
- [ ] [Deploy to Kubernetes, Amazon EC2, or Amazon ECS using Auto Deploy](https://docs.gitlab.com/ee/topics/autodevops/requirements.html)
- [ ] [Use pull-based deployments for improved Kubernetes management](https://docs.gitlab.com/ee/user/clusters/agent/)
- [ ] [Set up protected environments](https://docs.gitlab.com/ee/ci/environments/protected_environments.html)

***

# Editing this README

When you're ready to make this README your own, just edit this file and use the handy template below (or feel free to structure it however you want - this is just a starting point!).  Thank you to [makeareadme.com](https://www.makeareadme.com/) for this template.

## Suggestions for a good README
Every project is different, so consider which of these sections apply to yours. The sections used in the template are suggestions for most open source projects. Also keep in mind that while a README can be too long and detailed, too long is better than too short. If you think your README is too long, consider utilizing another form of documentation rather than cutting out information.

## Name
Choose a self-explaining name for your project.

## Description
Let people know what your project can do specifically. Provide context and add a link to any reference visitors might be unfamiliar with. A list of Features or a Background subsection can also be added here. If there are alternatives to your project, this is a good place to list differentiating factors.

## Badges
On some READMEs, you may see small images that convey metadata, such as whether or not all the tests are passing for the project. You can use Shields to add some to your README. Many services also have instructions for adding a badge.

## Visuals
Depending on what you are making, it can be a good idea to include screenshots or even a video (you'll frequently see GIFs rather than actual videos). Tools like ttygif can help, but check out Asciinema for a more sophisticated method.

## Installation
Within a particular ecosystem, there may be a common way of installing things, such as using Yarn, NuGet, or Homebrew. However, consider the possibility that whoever is reading your README is a novice and would like more guidance. Listing specific steps helps remove ambiguity and gets people to using your project as quickly as possible. If it only runs in a specific context like a particular programming language version or operating system or has dependencies that have to be installed manually, also add a Requirements subsection.

## Usage
Use examples liberally, and show the expected output if you can. It's helpful to have inline the smallest example of usage that you can demonstrate, while providing links to more sophisticated examples if they are too long to reasonably include in the README.

## Support
Tell people where they can go to for help. It can be any combination of an issue tracker, a chat room, an email address, etc.

## Roadmap
If you have ideas for releases in the future, it is a good idea to list them in the README.

## Contributing
State if you are open to contributions and what your requirements are for accepting them.

For people who want to make changes to your project, it's helpful to have some documentation on how to get started. Perhaps there is a script that they should run or some environment variables that they need to set. Make these steps explicit. These instructions could also be useful to your future self.

You can also document commands to lint the code or run tests. These steps help to ensure high code quality and reduce the likelihood that the changes inadvertently break something. Having instructions for running tests is especially helpful if it requires external setup, such as starting a Selenium server for testing in a browser.

## Authors and acknowledgment
Show your appreciation to those who have contributed to the project.

## License
For open source projects, say how it is licensed.

## Project status
If you have run out of energy or time for your project, put a note at the top of the README saying that development has slowed down or stopped completely. Someone may choose to fork your project or volunteer to step in as a maintainer or owner, allowing your project to keep going. You can also make an explicit request for maintainers.
