import sys
import os
root_dir = os.path.abspath(os.curdir)
sys.path.append(root_dir)
import subprocess

import h5py
import yaml
import src.g5505_utils as utils
import src.hdf5_vis as hdf5_vis
import src.hdf5_lib as hdf5_lib
import src.git_ops as git_ops
# TODO: incorporate lines 14-18 in git_ops module and refactor code where needed
current_branch_command = ['git','branch','--show-current']    
status_command = ['git','status']
add_command = lambda add_list: ['git','add'] + add_list
rm_command = lambda add_list: ['git','add'] + add_list
commit_command = lambda message: ['git','commit','-m', message]

#import input_files.config_file as config_file

import numpy as np



YAML_EXT = ".yaml"
TXT_EXT = ".txt"



def get_review_status(filename_path):

    filename_path_tail, filename_path_head = os.path.split(filename_path)
    filename, ext = os.path.splitext(filename_path_head)
    # TODO:    
    with open(os.path.join("review/",filename+"-review_status"+TXT_EXT),'r') as f:
        workflow_steps = []
        for line in f:
            workflow_steps.append(line)
    return workflow_steps[-1]

def parse_attribute(attr_value : dict):
    "Parse a dictionary attribute into an equivalent numpy structured array, which compatible with compound HDF5 type"
    dtype = []
    values_list = []
    max_length = max(len(str(attr_value[key])) for key in attr_value.keys())
    for key in attr_value.keys():
        if (not key=='rename_as'):
            dtype.append((key,f'S{max_length}'))
            values_list.append(attr_value[key])  

    if values_list:
        new_attr_value = np.array([tuple(values_list)],dtype=dtype)
    else:
        new_attr_value = 'missing'

    return new_attr_value

def convert_string_to_bytes(input_list: list):
    utf8_type = lambda max_length: h5py.string_dtype('utf-8', max_length)
    if input_list:
        max_length = max(len(item) for item in input_list)
        # Convert the strings to bytes with utf-8 encoding, specifying errors='ignore' to skip characters that cannot be encoded
        input_list_bytes = [item.encode('utf-8', errors='ignore') for item in input_list]
        input_array_bytes  = np.array(input_list_bytes,dtype=utf8_type(max_length)) 
    else:
        input_array_bytes  = np.array([],dtype=utf8_type(0))

    return input_array_bytes

def first_initialize_metadata_review(hdf5_file_path, reviewer_attrs, restart = False):

    """
    First: Initialize review branch with review folder with a copy of yaml representation of
    hdf5 file under review and by creating a txt file with the state of the review process, e.g., under review.
    
    """

    initials = reviewer_attrs['initials']
    #branch_name = '-'.join([reviewer_attrs['type'],'review_',initials])
    branch_name = '_'.join(['review',initials])

    hdf5_file_path_tail, filename_path_head = os.path.split(hdf5_file_path)
    filename, ext = os.path.splitext(filename_path_head)

    # Check file_path points to h5 file
    if not 'h5' in ext:
        raise ValueError("filename_path needs to point to an h5 file.")

    # Verify if yaml snapshot of input h5 file exists     
    if not os.path.exists(os.path.join(hdf5_file_path_tail,filename+YAML_EXT)):
        raise ValueError("metadata review cannot be initialized. The associated .yaml file under review was not found. Run take_yml_snapshot_of_hdf5_file(filename_path) ")

    # Initialize metadata review workflow
    # print("Create branch metadata-review-by-"+initials+"\n")

    #checkout_review_branch(branch_name)
    
    # Check you are working at the right branch   
    current_branch_command = ['git','branch','--show-current']
    curr_branch = subprocess.run(current_branch_command,capture_output=True,text=True,check=True)
    if not branch_name in curr_branch.stdout:
        raise ValueError("Branch "+branch_name+" was not found. \nPlease open a Git Bash Terminal, and follow the below instructions: \n1. Change directory to your project's directory. \n2. Excecute the command: git checkout "+branch_name)
        
    # Check if review file already exists and then check if it is still untracked
    review_yaml_file_path = os.path.join("review/",filename+YAML_EXT)
    review_yaml_file_path_tail, ext = os.path.splitext(review_yaml_file_path)    
    review_status_yaml_file_path = os.path.join(review_yaml_file_path_tail+"-review_status"+".txt")

    if not os.path.exists(review_yaml_file_path) or restart:
        review_yaml_file_path = utils.make_file_copy(os.path.join(hdf5_file_path_tail,filename+YAML_EXT), 'review') 
        if restart:
            print('metadata review has been reinitialized. The review files will reflect the current state of the hdf5 files metadata')

        

    #if not os.path.exists(os.path.join(review_yaml_file_path_tail+"-review_status"+".txt")):
        
        with open(review_status_yaml_file_path,'w') as f:
            f.write('under review')  

    # Stage untracked review files and commit them to local repository
    status = subprocess.run(status_command,capture_output=True,text=True,check=True)
    untracked_files = []
    for line in status.stdout.splitlines():
        #tmp = line.decode("utf-8")
            #modified_files.append(tmp.split()[1])    
        if 'review/' in line:
            if not 'modified' in line: # untracked filesand 
                untracked_files.append(line.strip())
            else: 
                untracked_files.append(line.strip().split()[1])

        if 'output_files/'+filename+YAML_EXT in line and not 'modified' in line:
            untracked_files.append(line.strip())

    if untracked_files:
        result = subprocess.run(add_command(untracked_files),capture_output=True,check=True)
        message = 'Initialized metadata review.'
        commit_output = subprocess.run(commit_command(message),capture_output=True,check=True)

        for line in commit_output.stdout.splitlines():
            print(line.decode('utf-8'))
    #else:
    #    print('This action will not have any effect because metadata review process has been already initialized.')




    #status_dict = repo_obj.status()
    #for filepath, file_status in status_dict.items():
        # Identify keys associated to review files and stage them
    #    if 'review/'+filename in filepath:
            # Stage changes
    #        repo_obj.index.add(filepath)

    #author = config_file.author #default_signature
    #committer = config_file.committer
    #message = "Initialized metadata review process."
    #tree = repo_obj.index.write_tree()
    #oid = repo_obj.create_commit('HEAD', author, committer, message, tree, [repo_obj.head.peel().oid])

    #print("Add and commit"+"\n") 

    return review_yaml_file_path, review_status_yaml_file_path

    
        
def second_save_metadata_review(review_yaml_file_path, reviewer_attrs):
    """
    Second: Once you're done reviewing the yaml representation of hdf5 file in review folder. 
    Change the review status to complete and save (add and commit) modified .yalm and .txt files in the project by
    running this function. 

    """
    # 1 verify review initializatin was performed first
    # 2. change review status in txt to complete
    # 3. git add review/ and git commit -m "Submitted metadata review"

    initials = reviewer_attrs['initials']
    #branch_name = '-'.join([reviewer_attrs['type'],'review','by',initials])
    branch_name = '_'.join(['review',initials])
    # TODO: replace with subprocess + git
    #checkout_review_branch(repo_obj, branch_name)

    # Check you are working at the right branch    
    curr_branch = subprocess.run(current_branch_command,capture_output=True,text=True,check=True)
    if not branch_name in curr_branch.stdout:
        raise ValueError('Please checkout ' + branch_name + ' via Git Bash before submitting metadata review files. ')

    # Collect modified review files
    status = subprocess.run(status_command,capture_output=True,check=True)
    modified_files = []
    os.path.basename(review_yaml_file_path)
    for line in status.stdout.splitlines():
        # conver line from bytes to str
        tmp = line.decode("utf-8")
        if 'modified' in tmp and 'review/' in tmp and os.path.basename(review_yaml_file_path) in tmp:
            modified_files.append(tmp.split()[1])         
    
    # Stage modified files and commit them to local repository    
    review_yaml_file_path_tail, review_yaml_file_path_head = os.path.split(review_yaml_file_path)
    filename, ext = os.path.splitext(review_yaml_file_path_head)
    if modified_files:
        review_status_file_path = os.path.join("review/",filename+"-review_status"+TXT_EXT)
        with open(review_status_file_path,'a') as f:
            f.write('\nsubmitted')

        modified_files.append(review_status_file_path)

        result = subprocess.run(add_command(modified_files),capture_output=True,check=True)
        message = 'Submitted metadata review.'
        commit_output = subprocess.run(commit_command(message),capture_output=True,check=True)

        for line in commit_output.stdout.splitlines():
            print(line.decode('utf-8'))
    else:
        print('Nothing to commit.')

#
def load_yaml(yaml_review_file):
    with open(yaml_review_file, 'r') as stream:
        try:
            return yaml.load(stream, Loader=yaml.FullLoader)
        except yaml.YAMLError as exc:
            print(exc)
            return None

def update_hdf5_attributes(input_hdf5_file, yaml_dict):

    def update_attributes(hdf5_obj, yaml_obj):
        for attr_name, attr_value in yaml_obj['attributes'].items():

            if not isinstance(attr_value, dict):
                attr_value = {'rename_as': attr_name, 'value': attr_value}
            
            if (attr_name in hdf5_obj.attrs.keys()): # delete or update
                if attr_value.get('delete'): # delete when True
                    hdf5_obj.attrs.__delitem__(attr_name)                        
                elif not (attr_value.get('rename_as') == attr_name): # update when true
                    hdf5_obj.attrs[attr_value.get('rename_as')] = hdf5_obj.attrs[attr_name] # parse_attribute(attr_value)  
                    hdf5_obj.attrs.__delitem__(attr_name)  
            else: # add a new attribute
                hdf5_obj.attrs.update({attr_name : parse_attribute(attr_value)})  

    with h5py.File(input_hdf5_file, 'r+') as f:
        for key in yaml_dict.keys():
            hdf5_obj = f[key]
            yaml_obj = yaml_dict[key]
            update_attributes(hdf5_obj, yaml_obj)

def update_hdf5_file_with_review(input_hdf5_file, yaml_review_file):
    yaml_dict = load_yaml(yaml_review_file)
    update_hdf5_attributes(input_hdf5_file, yaml_dict)
    # Regenerate yaml snapshot of updated HDF5 file
    output_yml_filename_path = hdf5_vis.take_yml_snapshot_of_hdf5_file(input_hdf5_file)
    print(f'{output_yml_filename_path} was successfully regenerated from the updated version of{input_hdf5_file}')
    
def third_update_hdf5_file_with_review(input_hdf5_file, yaml_review_file, reviewer_attrs={}, hdf5_upload=False):
    if 'submitted' not in get_review_status(input_hdf5_file):
        raise ValueError('Review yaml file must be submitted before trying to perform an update. Run first second_submit_metadata_review().')

    update_hdf5_file_with_review(input_hdf5_file, yaml_review_file)
    git_ops.perform_git_operations(hdf5_upload)

def count(hdf5_obj,yml_dict):
    print(hdf5_obj.name)
    if isinstance(hdf5_obj,h5py.Group) and len(hdf5_obj.name.split('/')) <= 4:
        obj_review = yml_dict[hdf5_obj.name]
        additions = [not (item in hdf5_obj.attrs.keys()) for item in obj_review['attributes'].keys()]
        count_additions = sum(additions)
        deletions = [not (item in obj_review['attributes'].keys()) for item in hdf5_obj.attrs.keys()]
        count_delections = sum(deletions)
        print('additions',count_additions, 'deletions', count_delections)

def last_submit_metadata_review(reviewer_attrs):

    """Fourth: """

    initials =reviewer_attrs['initials']

    repository = 'origin'
    branch_name = '_'.join(['review',initials])

    push_command = lambda repository,refspec: ['git','push',repository,refspec]
    current_branch_command = ['git','branch','--show-current']
    list_branches_command = ['git','branch','--list']

    branches = subprocess.run(list_branches_command,capture_output=True,text=True,check=True)
    if not branch_name in branches.stdout:
        print('There is no branch named '+branch_name+'.\n')
        print('Make sure to run data owner review workflow from the beginning without missing any steps.')
        return

    curr_branch = subprocess.run(current_branch_command,capture_output=True,text=True,check=True)
    if not branch_name in curr_branch.stdout:
        print('Complete metadata review could not be completed.\n')
        print('Make sure a data-owner workflow has already been started on branch '+branch_name+'\n')
        print('The step "Complete metadata review" will have no effect.')
        return

    
 
    # push
    result = subprocess.run(push_command(repository,branch_name),capture_output=True,text=True,check=True)
    print(result.stdout)

    # 1. git add output_files/
    # 2. delete review/
    #shutil.rmtree(os.path.join(os.path.abspath(os.curdir),"review"))
    # 3. git rm review/
    # 4. git commit -m "Completed review process. Current state of hdf5 file and yml should be up to date."
    return result.returncode


#import config_file
#import hdf5_vis

def main():    

    output_filename_path = "output_files/unified_file_smog_chamber_2024-03-19_UTC-OFST_+0100_NG.h5"
    output_yml_filename_path = "output_files/unified_file_smog_chamber_2024-03-19_UTC-OFST_+0100_NG.yalm"
    output_yml_filename_path_tail, filename = os.path.split(output_yml_filename_path)
    #output_yml_filename_path = hdf5_vis.take_yml_snapshot_of_hdf5_file(output_filename_path)

    #first_initialize_metadata_review(output_filename_path,initials='NG')
    #second_submit_metadata_review()
    #if os.path.exists(os.path.join(os.path.join(os.path.abspath(os.curdir),"review"),filename)):
     #   third_update_hdf5_file_with_review(output_filename_path, os.path.join(os.path.join(os.path.abspath(os.curdir),"review"),filename))
    #fourth_complete_metadata_review()    

#if __name__ == '__main__':

#    main()
